#!/usr/bin/env python

"""
Make dummy data and fixtures and stuff.
"""

import collections
import json
import hashlib
import random
import socket
import struct
from pynsot import client
try:
    import faker
except ImportError:
    print "The 'fake-factory' module is required! (Hint: pip install fake-factory)"
    raise


# Constants and stuff
fake = faker.Factory.create()

# Dummy config data used for testing dotfile and client
CONFIG_DATA = {
    'email': 'jathan@localhost',
    'url': 'http://localhost:8990/api',
    'auth_method': 'auth_token',
    'secret_key': 'MJMOl9W7jqQK3h-quiUR-cSUeuyDRhbn2ca5E31sH_I=',
}

# Payload used to create Network & Device attributes used for testing.
TEST_ATTRIBUTES = {
    'attributes': [
        {
            'name': 'cluster',
            'resource_name': 'Device',
            'description': 'Device cluster.',
            'constraints': {'allow_empty': True},
        },
        {
            'name': 'foo',
            'resource_name': 'Device',
            'description': 'Foo for Devices.',
        },
        {
            'name': 'owner',
            'resource_name': 'Device',
            'description': 'Device owner.',
        },
        {
            'name': 'cluster',
            'resource_name': 'Network',
            'description': 'Network cluster.',
            'constraints': {'allow_empty': True},
        },
        {
            'name': 'foo',
            'resource_name': 'Network',
            'description': 'Foo for Networks.',
        },
        {
            'name': 'owner',
            'resource_name': 'Network',
            'description': 'Network owner.',
        },
    ]
}

# Phony attributes to randomly generate for testing.
ATTRIBUTE_DATA = {
    'cluster': ['', 'lax', 'iad', 'sjc'],
    'owner': ['jathan', 'bob', 'alice'],
    'foo': ['bar', 'baz', 'spam'],
}

# Used to store Attribute/value pairs
Attribute = collections.namedtuple('Attribute', 'name value')


def generate_attributes(attributes=None, as_dict=True):
    """
    Randomly choose attributes and values for testing.

    :param attributes:
        Dictionary of attribute names and values

    :param as_dict:
        If set return a dict vs. list of Attribute objects
    """
    if attributes is None:
        attributes = ATTRIBUTE_DATA
    attrs = []
    for attr_name, attr_values in attributes.iteritems():
        if random.choice((True, False)):
            attr_value = random.choice(attr_values)
            attrs.append(Attribute(attr_name, attr_value))
    if as_dict:
        attrs = dict(attrs)
    return attrs


def generate_crap(num_items=100):
    """
    Return a list of dicts of {name: description}.

    :param num_items:
        Number of items to generate
    """
    names = set()
    while len(names) < num_items:
        names.add(fake.word().title())
    data = []
    for name in names:
        description = ' '.join(fake.words()).capitalize()
        item = {'name': name, 'description': description}
        data.append(item)
    return data


def populate_sites(site_data):
    """Populate sites from fixture data."""
    api = client.Client('http://localhost:8990/api', email='jathan@localhost')
    results = []
    for d in site_data:
        try:
            result = api.sites.post(d)
        except Exception as err:
            print err, d['name']
        else:
            results.append(result)
    print 'Created', len(results), 'sites.'


def generate_words(num_items=100, title=False, add_suffix=False):
    stuff = set()
    while len(stuff) < num_items:
        things = (fake.word(), fake.first_name(), fake.last_name())
        suffix = str(random.randint(1, 8)) if add_suffix else ''
        word = random.choice(things) + suffix
        if title:
            word = word.title()
        else:
            word = word.lower()
        stuff.add(word)
    return sorted(stuff)


def generate_ipv4():
    """Generate a random IPv4 address."""
    return socket.inet_ntoa(struct.pack('>I', random.randint(1, 0xffffffff)))


def generate_ipv4list(num_items=100, include_hosts=False):
    """
    Generate a list of unique IPv4 addresses. This is a total hack.

    :param num_items:
        Number of items to generate

    :param include_hosts:
        Whether to include /32 addresses
    """
    ipset = set()
    # Keep iterating and hack together cidr prefixes if we detect empty
    # trailing octects. This is so lame that we'll mostly just end up with a
    # bunch of /24 networks.
    while len(ipset) < num_items:
        ip = generate_ipv4()
        if ip.endswith('.0.0.0'):
            prefix = '/8'
        elif ip.endswith('.0.0'):
            prefix = '/16'
        elif ip.endswith('.0'):
            prefix = '/24'
        elif include_hosts:
            prefix = '/32'
        else:
            continue

        ip += prefix
        ipset.add(ip)
    return sorted(ipset)


def generate_hostnames(num_items=100):
    """
    Generate a random list of hostnames.

    :param num_items:
        Number of items to generate
    """
    hostnames = generate_words(num_items, add_suffix=True)
    return hostnames


def generate_devices(num_items=100, with_attributes=True):
    """
    Return a list of dicts for Device creation.

    :param num_items:
        Number of items to generate

    :param with_attributes:
        Whether to include Attributes
    """
    hostnames = generate_hostnames(num_items)

    devices = []
    for hostname in hostnames:
        item = {'hostname': hostname}
        if with_attributes:
            attributes = generate_attributes()
            item['attributes'] = attributes

        devices.append(item)
    return {'devices': devices}


def generate_networks(num_items=100, with_attributes=True):
    """
    Return a list of dicts for Network creation.

    :param num_items:
        Number of items to generate

    :param with_attributes:
        Whether to include Attributes
    """
    ipv4list = generate_ipv4list(num_items)

    networks = []
    for cidr in ipv4list:
        attributes = generate_attributes()
        item = {'cidr': cidr}
        if with_attributes:
            attributes = generate_attributes()
            item['attributes'] = attributes

        networks.append(item)
    return {'networks': networks}



# -*- coding: utf-8 -*-

"""
Sub-command for Attributes.

In all cases ``data = ctx.params`` when calling the appropriate action method
on ``ctx.obj``. (e.g. ``ctx.obj.add(ctx.params)``)

Also, ``action = ctx.info_name`` *might* reliably contain the name of the
action function, but still not sure about that. If so, every function could be
fundamentally simplified to this::

    getattr(ctx.obj, ctx.info_name)(ctx.params)
"""

__author__ = 'Jathan McCollum'
__maintainer__ = 'Jathan McCollum'
__email__ = 'jathan@dropbox.com'
__copyright__ = 'Copyright (c) 2015 Dropbox, Inc.'


import click
from . import callbacks


# Ordered list of 2-tuples of (field, display_name) used to translate object
# field names oto their human-readable form when calling .print_list().
DISPLAY_FIELDS = (
    ('id', 'ID'),
    ('name', 'Name'),
    ('resource_name', 'Resource'),
    # ('site_id': 'Site ID'),
    ('required', 'Required?'),
    ('display', 'Display?'),
    ('multi', 'Multi?'),
    ('description', 'Description'),
)

# Fields to display when viewing a single record.
VERBOSE_FIELDS = (
    ('name', 'Name'),
    ('resource_name', 'Resource'),
    ('required', 'Required?'),
    ('display', 'Display?'),
    ('multi', 'Multi?'),
    ('constraints', 'Constraints'),
    ('description', 'Description'),
)

# Sub-fields for Attribute constraints
CONSTRAINT_FIELDS = ('allow_empty', 'pattern', 'valid_values')


# Main group
@click.group()
@click.pass_context
def cli(ctx):
    """
    Attribute objects.

    Attributes are arbitrary key/value pairs that can be assigned to various
    resources. If an attribute is required then additions/updates for that
    resource will require that attribute be present. Existing resources will
    not be forcefully validated until update.
    """


# Add
@cli.command()
@click.option(
    '--allow-empty',
    help='Whether to allow this Attribute to have an empty value.',
    is_flag=True,
)
@click.option(
    '-b',
    '--bulk-add',
    metavar='FILENAME',
    help='Bulk add Attributes from the specified colon-delimited file.',
    type=click.File('rb'),
    callback=callbacks.process_bulk_add,
)
@click.option(
    '-d',
    '--description',
    metavar='DESC',
    help='A helpful description of the Attribute.',
)
@click.option(
    '--display',
    help='Whether this Attribute should be be displayed by default in UIs.',
    is_flag=True,
)
@click.option(
    '--multi',
    help='Whether the attribute should be treated as a list type.',
    is_flag=True,
)
@click.option(
    '-n',
    '--name',
    metavar='NAME',
    help='The name of the Attribute.  [required]',
)
@click.option(
    '-p',
    '--pattern',
    help='Constrain attribute values to this regex pattern.',
    default='',
)
@click.option(
    '--required',
    help='Whether this Attribute should be required.',
    is_flag=True,
)
@click.option(
    '-r',
    '--resource-name',
    metavar='RESOURCE',
    help='The resource type this Attribute is for (e.g. Device).  [required]',
    callback=callbacks.transform_resource_name,
)
@click.option(
    '-s',
    '--site-id',
    metavar='SITE_ID',
    type=int,
    help='Unique ID of the Site this Attribute is under.  [required]',
    callback=callbacks.process_site_id,
)
@click.option(
    '-V',
    '--valid-values',
    metavar='VALUE',
    help='Valid values for this Attribute. May be specified multiple times.',
    multiple=True,
)
@click.pass_context
def add(ctx, allow_empty, bulk_add, description, display, multi, name, pattern,
        resource_name, required, valid_values, site_id):
    """
    Add a new Attribute.

    You must provide a Site ID using the -s/--site-id option.

    When adding a new Attribute, you must provide a value for the -n/--name
    and -r/--resource-name options.
    """
    data = bulk_add or ctx.params

    # Enforce required options
    if bulk_add is None:
        if name is None:
            raise click.UsageError('Missing option "-n" / "--name".')
        if resource_name is None:
            raise click.UsageError('Missing option "-r" / "--resource-name".')

    # Handle the constraint fields
    data = callbacks.process_constraints(
        data, constraint_fields=CONSTRAINT_FIELDS
    )
    ctx.obj.add(data)


# List
@cli.command()
@click.option(
    '-i',
    '--id',
    metavar='ID',
    help='Unique ID of the Attribute being retrieved.',
)
@click.option(
    '--display/--no-display',
    help='Filter to Attributes meant to be displayed.',
    default=None,
)
@click.option(
    '-l',
    '--limit',
    metavar='LIMIT',
    type=int,
    help='Limit result to N resources.',
)
@click.option(
    '--multi/--no-multi',
    help='Filter on whether Attributes are list type or not.',
    default=None,
)
@click.option(
    '-n',
    '--name',
    metavar='NAME',
    help='Filter to Attribute with this name.',
)
@click.option(
    '-o',
    '--offset',
    metavar='OFFSET',
    type=int,
    help='Skip the first N resources.',
)
@click.option(
    '--required/--no-required',
    help='Filter to Attributes that are required.',
    default=None,
)
@click.option(
    '-r',
    '--resource-name',
    metavar='RESOURCE',
    help='Filter to Attributes for a specific resource (e.g. Network)',
    callback=callbacks.transform_resource_name,
)
@click.option(
    '-s',
    '--site-id',
    metavar='SITE_ID',
    type=int,
    help='Unique ID of the Site this Attribute is under.  [required]',
    callback=callbacks.process_site_id,
)
@click.pass_context
def list(ctx, id, display, limit, multi, name, offset, required, resource_name,
         site_id):
    """
    List existing Attributes for a Site.

    You must provide a Site ID using the -s/--site-id option.

    When listing Attributes, all objects are displayed by default. You may
    optionally lookup a single Attribute by name using the -n/--name option or
    by ID using the -i/--id option.

    You may limit the number of results using the -l/--limit option.
    """
    data = ctx.params

    # If we provide ID, be verbose
    if id is not None:
        display_fields = VERBOSE_FIELDS
    else:
        display_fields = DISPLAY_FIELDS

    ctx.obj.list(data, display_fields=display_fields)


# Remove
@cli.command()
@click.option(
    '-i',
    '--id',
    metavar='ID',
    type=int,
    help='Unique ID of the Attribute being deleted.',
    required=True,
)
@click.option(
    '-s',
    '--site-id',
    metavar='SITE_ID',
    type=int,
    help='Unique ID of the Site this Attribute is under.  [required]',
    callback=callbacks.process_site_id,
)
@click.pass_context
def remove(ctx, id, site_id):
    """
    Remove an Attribute.

    You must provide a Site ID using the -s/--site-id option.

    When removing an Attribute, you must provide the unique ID using -i/--id.
    You may retrieve the ID for an Attribute by looking it up by name for a
    given Site:

        nsot attributes list --name <name> --site <site_id>
    """
    data = ctx.params
    ctx.obj.remove(**data)


# Update
@cli.command()
@click.option(
    '--allow-empty/--no-allow-empty',
    help='Whether to allow this Attribute to have an empty value.',
)
@click.option(
    '-d',
    '--description',
    metavar='DESC',
    help='A helpful description of the Attribute.',
)
@click.option(
    '--display/--no-display',
    help='Whether this Attribute should be be displayed by default in UIs.',
    default=None,
)
@click.option(
    '-i',
    '--id',
    metavar='ID',
    type=int,
    help='Unique ID of the Attribute being updated.',
    required=True,
)
@click.option(
    '--multi/--no-multi',
    help='Whether the Attribute should be treated as a list type.',
    default=None,
)
@click.option(
    '-p',
    '--pattern',
    help='Constrain attribute values to this regex pattern.',
    default='',
)
@click.option(
    '--required/--no-required',
    help='Whether this Attribute should be required.',
    default=None,
)
@click.option(
    '-s',
    '--site-id',
    metavar='SITE_ID',
    type=int,
    help='Unique ID of the Site this Attribute is under.  [required]',
    callback=callbacks.process_site_id,
)
@click.option(
    '-V',
    '--valid-values',
    metavar='VALUE',
    help='Valid values for this Attribute. May be specified multiple times.',
    multiple=True,
)
@click.pass_context
def update(ctx, allow_empty, description, display, id, multi, pattern,
           required, site_id, valid_values):
    """
    Update an Attribute.

    You must provide a Site ID using the -s/--site-id option.

    When updating an Attribute you must provide the unique ID (-i/--id) and at
    least one of the optional arguments.
    """
    optional = (allow_empty, description, display, multi, pattern, required,
                valid_values)
    provided = []
    for opt in optional:
        if opt in (True, False) or isinstance(opt, basestring):
            provided.append(opt)

    # If none of them were provided, complain.
    if not provided:
        msg = 'You must supply at least one the optional arguments.'
        raise click.UsageError(msg)

    # Handle the constraint fields
    data = callbacks.process_constraints(
        ctx.params, constraint_fields=CONSTRAINT_FIELDS
    )

    ctx.obj.update(data)



# -*- coding: utf-8 -*-

"""
Specialized serializers for NSoT API client.

This is an example of how you would use this with the Client object, to make it
return objects instead of dicts::

    >>> serializer = ModelSerializer()
    >>> api = Client(url, serializer=serializer)
    >>> obj = api.sites(1).get()
    >>> obj
    <Site(id=1, description=u'Foo site', name=u'Foo')>
"""

__author__ = 'Jathan McCollum'
__maintainer__ = 'Jathan McCollum'
__email__ = 'jathan@dropbox.com'
__copyright__ = 'Copyright (c) 2015 Dropbox, Inc.'


from slumber.serialize import JsonSerializer
import models


class ModelSerializer(JsonSerializer):
    """This serializes to a model instead of a dict."""
    key = 'model'

    def get_serializer(self, *args, **kwargs):
        return self

    def loads(self, data):
        obj_data = super(ModelSerializer, self).loads(data)
        return models.ApiModel(obj_data)



# -*- coding: utf-8 -*-

"""
Model classes to represent API dict results as objects in an ORM style.

See the examples in the docstring for ``ApiModel``.
"""

__author__ = 'Jathan McCollum'
__maintainer__ = 'Jathan McCollum'
__email__ = 'jathan@dropbox.com'
__copyright__ = 'Copyright (c) 2015 Dropbox, Inc.'


import collections


# Valid top-level types.
TYPES = ('network', 'attribute', 'user', 'site', 'device')


# What an error response looks like:
'''
{
    "status": "error",
    "error": {
        "code": 404,
        "message": "Resource not found."
    }
}
'''


class ApiModel(collections.MutableMapping):
    """
    Simple class to make an API response dict into an object.

    For example, for a site object::

        >>> site1 = api.sites(1).get()
        >>> site1
        {u'data': {u'site': {u'description': u'Foo site', u'id': 1, u'name':
         u'Foo'}},
         u'status': u'ok'}
        >>> site_obj = ApiModel(site1)
        >>> site_obj
        <Site(id=1, name=u'Foo', description=u'Foo site')>
        >>> site_obj.name
        u'Foo'

    Network object::

        >>> site = api.sites(1).get()
        >>> nw1 = site.networks(1).get()
        >>> nw1 = api.sites(1).networks(1).get()
        {u'data': {u'network': {u'attributes': {},
           u'id': 1,
           u'ip_version': u'4',
           u'is_ip': False,
           u'network_address': u'10.0.0.0',
           u'parent_id': None,
           u'prefix_length': 8,
           u'site_id': 1}},
         u'status': u'ok'}
        >>> nw_obj = ApiModel(nw1)
        >>> nw_obj
        <Network(is_ip=False, network_address=u'10.0.0.0', site_id=1,
         parent_id=None, prefix_length=8, ip_version=u'4', attributes={},
         id=1)>
        >>> nw_obj.network_address
        u'10.0.0.0'
    """
    def __init__(self, data, **kwargs):
        if 'data' not in data:
            raise SyntaxError("Data must contain a 'data' field.")
        obj = data['data']

        if len(obj) > 1:
            raise SyntaxError('More than one object type found.')
        obj_type = obj.keys()[0]

        if obj_type not in TYPES:
            raise SyntaxError('Invalid type: %s' % (obj_type,))

        self._object_type = obj_type
        obj_data = obj[obj_type]
        self.__dict__.update(obj_data)

    def _repr_name(self):
        """Normalize object type to display in repr."""
        cls_name = self._object_type.title().replace('_', '')
        return cls_name

    def __repr__(self):
        cls_name = self._repr_name()
        attrs = ', '.join('%s=%r' % (k, v) for k, v in self.items())
        return '<%s(%s)>' % (cls_name, attrs)

    # collections.MutableMapping requires these to be defined to dictate the
    # dictionary-like behavior.

    def __getitem__(self, val):
        return self.__dict__[val]

    def __setitem__(self, key, val):
        self.__dict__[key] = val

    def __delitem__(self, key):
        del self.__dict__[key]

    def __iter__(self):
        return iter(k for k in self.__dict__ if not k.startswith('_'))

    def __len__(self):
        return len(self.__dict__)



# -*- coding: utf-8 -*-

"""
Sub-command for Sites.

In all cases ``data = ctx.params`` when calling the appropriate action method
on ``ctx.obj``. (e.g. ``ctx.obj.add(ctx.params)``)

Also, ``action = ctx.info_name`` *might* reliably contain the name of the
action function, but still not sure about that. If so, every function could be
fundamentally simplified to this::

    getattr(ctx.obj, ctx.info_name)(ctx.params)
"""

__author__ = 'Jathan McCollum'
__maintainer__ = 'Jathan McCollum'
__email__ = 'jathan@dropbox.com'
__copyright__ = 'Copyright (c) 2015 Dropbox, Inc.'


import click


# Ordered list of 2-tuples of (field, display_name) used to translate object
# field names oto their human-readable form when calling .print_list().
DISPLAY_FIELDS = (
    ('id', 'ID'),
    ('name', 'Name'),
    ('description', 'Description'),
)


# Main group
@click.group()
@click.pass_context
def cli(ctx):
    """
    Site objects.

    Sites are the top-level resource from which all other resources descend. In
    other words, Sites contain Attributes, Changes, Devices, and Networks.

    Sites function as unique namespaces that can contain other resources. Sites
    allow an organization to have multiple instances of potentially conflicting
    resources. This could be beneficial for isolating corporeate vs. production
    environments, or pulling in the IP space of an acquisition.
    """
    # if ctx.obj.verbose:
    #     print 'I am:', ctx.info_name
    #     print 'my parent is:', ctx.parent.info_name


# Add
@cli.command()
@click.option(
    '-d',
    '--description',
    metavar='DESC',
    help='A helpful description for the Site.',
)
@click.option(
    '-n',
    '--name',
    metavar='NAME',
    help='The name of the Site.',
    required=True,
)
@click.pass_context
def add(ctx, description, name):
    """
    Add a new Site.

    When adding a Site, you must provide a name using the -n/--name option.

    You may provide a helpful description for the Site using the
    -d/--description option.
    """
    data = ctx.params
    ctx.obj.add(data)


# List
@cli.command()
@click.option(
    '-i',
    '--id',
    metavar='ID',
    type=int,
    help='Unique ID of the Site to retrieve.',
)
@click.option(
    '-l',
    '--limit',
    metavar='LIMIT',
    type=int,
    help='Limit results to N resources.',
)
@click.option(
    '-n',
    '--name',
    metavar='NAME',
    help='Filter by Site name.'
)
@click.option(
    '-o',
    '--offset',
    metavar='OFFSET',
    type=int,
    help='Skip the first N resources.',
)
@click.pass_context
def list(ctx, id, limit, name, offset):
    """
    List existing Sites.

    When listing Sites, all objects are displayed by default. You may
    optionally lookup a single Site by name using the -n/--name option or by ID
    using the -i/--id option.

    You may limit the number of results using the -l/--limit option.

    NOTE: The -i/--id option will take precedence causing all other options
    to be ignored.
    """
    data = ctx.params
    ctx.obj.list(data, display_fields=DISPLAY_FIELDS)


# Remove
@cli.command()
@click.option(
    '-i',
    '--id',
    metavar='ID',
    type=int,
    help='Unique ID of the Site that should be removed.',
    required=True,
)
@click.pass_context
def remove(ctx, id):
    """
    Remove a Site.

    When removing a site, you must provide the unique ID using -i/--id. You may
    retrieve the ID for a Site by looking it up by name using:

        nsot sites list --name <name>
    """
    data = ctx.params
    ctx.obj.remove(**data)


# Update
@cli.command()
@click.option(
    '-d',
    '--description',
    metavar='DESC',
    help='A helpful description for the Site.',
)
@click.option(
    '-i',
    '--id',
    metavar='ID',
    type=int,
    help='Unique ID of the Site that should be updated.',
    required=True,
)
@click.option(
    '-n',
    '--name',
    metavar='NAME',
    help='The name of the Site.',
)
@click.pass_context
def update(ctx, description, id, name):
    """
    Update a Site.

    When updating a Site you must provide the unique ID (-i/--id) and at least
    one of the -n/--name or -d/--description arguments.
    """
    if name is None and description is None:
        msg = 'You must supply at least one of -n/--name or -d/--description'
        raise click.UsageError(msg)

    data = ctx.params
    ctx.obj.update(data)



"""
Test the dotfile.
"""

import logging
import os
import tempfile
import unittest

from pynsot import dotfile

from .fixtures import CONFIG_DATA


class TestDotFile(unittest.TestCase):
    def setUp(self):
        """Automatically create a tempfile for each test."""
        fd, filepath = tempfile.mkstemp()
        self.filepath = filepath
        self.config_data = CONFIG_DATA.copy()

    def test_read_success(self):
        """Test that file can be read."""
        config = dotfile.Dotfile(self.filepath)
        config.write(self.config_data)
        config.read()

    def test_read_failure(self):
        """Test that that a missing file raises an error."""
        self.filepath = tempfile.mktemp()  # Doesn't create the temp file
        config = dotfile.Dotfile(self.filepath)
        self.assertRaises(
            IOError,  # This means it's trying to read from stdin
            config.read,
        )

    def test_write(self):
        """Test that file can be written."""
        config = dotfile.Dotfile(self.filepath)
        config.write(self.config_data)

    def test_validate_perms_success(self):
        """Test that file permissions are ok."""
        self.filepath = tempfile.mktemp()  # Doesn't create a temp file
        config = dotfile.Dotfile(self.filepath)
        config.write({})
        config.validate_perms()

    def test_validate_fields(self):
        """Test that fields check out."""
        config = dotfile.Dotfile(self.filepath)

        # We're going to test every field.
        fields = sorted(self.config_data)
        my_config = {}

        # Iterate through the sorted list of fields to make sure that each one
        # raises an error as expected.
        err = 'Missing required field: '
        for field in fields:
            with self.assertRaisesRegexp(
                dotfile.DotfileError,
                err + field
            ):
                config.read()

            my_config[field] = self.config_data[field]
            config.write(my_config)

    def tearDown(self):
        """Delete a tempfile if it exists. Otherwise carry on."""
        try:
            os.remove(self.filepath)
        except OSError:
            pass



#!/usr/bin/env python

import os
from setuptools import find_packages, setup

execfile('pynsot/version.py')

with open('requirements.txt') as requirements:
    required = requirements.read().splitlines()

kwargs = {
    'name': 'pynsot',
    'version': str(__version__),
    'packages': find_packages(exclude=['tests']),
    'description': 'Python interface for Network Source of Truth (nsot)',
    'author': 'Jathan McCollum',
    'maintainer': 'Jathan McCollum',
    'author_email': 'jathan@dropbox.com',
    'maintainer_email': 'jathan@dropbox.com',
    'license': 'Apache',
    'install_requires': required,
    'url': 'https://github.com/dropbox/pynsot',
    'entry_points': """
        [console_scripts]
        nsot=pynsot.app:app
        snot=pynsot.app:app
    """,
    'classifiers': [
        'Programming Language :: Python',
        'Topic :: Software Development',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ]
}

setup(**kwargs)



"""
Tests for 'site' objects.

NOT YET IMPLEMENTED
"""

import json
import requests
import responses
import unittest


email = 'jathan@localhost'
BASE_URL = 'http://localhost:8990/api'
secret_key = 'MJMOl9W7jqQK3h-quiUR-cSUeuyDRhbn2ca5E31sH_I='


ADD_PAYLOAD_SUCCESS = {'name': 'Elvis', 'description': 'Presley'}
ADD_RESPOND_SUCCESS = {u'data': {u'site': {u'description': u'Presley', u'id': 2, u'name': u'Elvis'}}, u'status': u'ok'}
ADD_PAYLOAD_FAILURE = {'name': '', 'description': ''}
ADD_RESPOND_FAILURE = {u'error': {u'code': 400, u'message': u'Name is a required field.'}, u'status': u'error'}


class TestSites(unittest.TestCase):
    def setUp(self):
        self.base_url = BASE_URL + '/sites'
        self.content_type = 'application/json'
        self.headers = {'content-type': self.content_type}

    @responses.activate
    def test_add_success(self):
        """Test successful addition of a Site."""
        responses.add(
            responses.POST,
            self.base_url,
            body=json.dumps(ADD_RESPOND_SUCCESS),
            status=200,
            content_type=self.content_type
        )

        # Create the Site
        resp = requests.post(
            self.base_url,
            data=json.dumps(ADD_PAYLOAD_SUCCESS),
            headers=self.headers
        )

        # Verify success!
        self.assertEqual(resp.json(), ADD_RESPOND_SUCCESS)
        self.assertEqual(responses.calls[0].request.url, self.base_url)
        self.assertEqual(
            responses.calls[0].response.text,
            json.dumps(ADD_RESPOND_SUCCESS)
        )

    '''
    def test_add_failure():
        pass

    def test_list():
        pass

    def test_remove():
        pass

    def test_update():
        pass
    '''
